package net.smart.core;

import org.objectweb.asm.*;

public class SmartCoreMethodVisitor extends MethodVisitor
{
	private final SmartCoreTransformation transformation;

	public SmartCoreMethodVisitor(MethodVisitor paramMethodVisitor, SmartCoreTransformation transformation)
	{
		super(262144, paramMethodVisitor);

		this.transformation = transformation;

		if(transformation.addBeforeHook)
			invokeHook("before");
	}

	@Override
	public void visitInsn(int opcode)
	{
		if(transformation.addAfterHook && (opcode == Opcodes.RETURN || opcode == Opcodes.IRETURN || opcode == Opcodes.LRETURN ||opcode == Opcodes.FRETURN || opcode == Opcodes.DRETURN || opcode == Opcodes.ARETURN))
			invokeHook("after");
		super.visitInsn(opcode);
	}

	@SuppressWarnings("deprecation")
	private void invokeHook(String prefix)
	{
		if(transformation.addThisAsFirstParameter)
			mv.visitVarInsn(Opcodes.ALOAD, 0);
		for(int i=0; transformation.parameterTypeNames != null && i<transformation.parameterTypeNames.length; i++)
			if(transformation.isHookParameter(i))
				mv.visitVarInsn(Opcodes.ALOAD, i + 1);
		mv.visitMethodInsn(Opcodes.INVOKESTATIC, transformation.getHookClassDesc(), transformation.getHookName(prefix), transformation.getHookMethodDesc());
	}
}